//+------------------------------------------------------------------+
//|                                                    SafeArray.mqh |
//|                                 Copyright  2011-2016, Marketeer |
//|                          https://www.mql5.com/en/users/marketeer |
//| MQL's OOP notes: rubber arrays, safe numbers,                    |
//|                     exceptions handling and other tricks: part 1 |
//|                        https://www.mql5.com/en/blogs/post/683241 |
//+------------------------------------------------------------------+

template <typename T>
class RubbArray;

template <typename T>
class Editor
{
  RubbArray<T> *parent;
  int index;
  
  public:
    Editor(RubbArray<T> &array)
    {
      parent = &array;
    }
    Editor<T> *operator[](int i)
    {
      index = i;
      return &this;
    }
    void operator=(T value)
    {
      if(index != -1)
      {
        parent.change(index, value);
      }
    }
};

template <typename T>
class RubbArray
{
  private:
    Editor<T> *editor;
    
  protected:
    T data[];

    void add(T d)
    {
      int n = ArraySize(data);
      ArrayResize(data, n + 1);
      data[n] = d;
    }

  public:
    void change(int i, T value)
    {
      data[i] = value;
    }
    
    RubbArray()
    {
      editor = new Editor<T>(this);
    }
    
    ~RubbArray()
    {
      delete editor;
    }
  
    T operator[](int i) const
    {
      if(i < 0 || i >= ArraySize(data))
      {
        #ifdef EXCEPTIONS
          return try::throw("Read index out of bounds: array size=" + (string)ArraySize(data) + ", index=" + (string)i);
        #else
          return 0;
        #endif
      }
      return data[i];
    }

    Editor<T> *operator[](uint i)
    {
      return edit(i);
    }
    
    Editor<T> *edit(int i)
    {
      if(i < 0 || i >= ArraySize(data))
      {
        #ifdef EXCEPTIONS
        /* drop */try::throw("Write index out of bounds: array size=" + (string)ArraySize(data) + ", index=" + (string)i);
        #endif
        return editor[-1];
      }
      return editor[i];
    }

    RubbArray *operator<<(T d)
    {
      add(d);
      return &this;
    }

    T operator=(T d)
    {
      add(d);
      return d;
    }

    void operator=(const RubbArray &d)
    {
      int i, n = d.size();
      ArrayResize(data, n);
      for(i = 0; i < n; i++)
      {
        data[i] = d[i];
      }
    }

    RubbArray *operator+(const RubbArray &d)
    {
      int n = d.size();
      for(int i = 0; i < n; i++)
      {
        add(d[i]);
      }
      return &this;
    }

    T operator>>(int i)
    {
      T d = this[i];
      if(i >= 0 || i < ArraySize(data))
      {
        ArrayCopy(data, data, i, i + 1);
        ArrayResize(data, ArraySize(data) - 1);
      }
      return d;
    }
    
    int size() const
    {
      return ArraySize(data);
    }
    
    string toString() const
    {
      int n = ArraySize(data);
      string s;
      for(int i = 0; i < n; i++)
      {
        s += (string)data[i] + ",";
      }
      return s;
    }
};

#define edit(A) ((uint)A)
